<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2020 Sword SSl
 *  All rights reserved
 *
 ***************************************************************/
define('FIELD_ITEMS', 'amountOfItems');

/**
 * tx_mksearch_scheduler_IndexTaskAddFieldProvider.
 */
class tx_mksearch_scheduler_IndexTaskAddFieldProvider extends Tx_Rnbase_Scheduler_FieldProvider
{
    /**
     * This method is used to define new fields for adding or editing a task
     * In this case, it adds an email field.
     *
     * @param array                                                     $taskInfo:        reference to the array containing the info used in the add/edit form
     * @param Tx_Rnbase_Scheduler_Task                                  $task:            when editing, reference to the current task object. Null when adding.
     * @param \TYPO3\CMS\Scheduler\Controller\SchedulerModuleController $schedulerModule: reference to the calling object (Scheduler's BE module)
     *
     * @return array Array containg all the information pertaining to the additional fields
     *               The array is multidimensional, keyed to the task class name and each field's id
     *               For each field it provides an associative sub-array with the following:
     *               ['code']        => The HTML code for the field
     *               ['label']       => The label of the field (possibly localized)
     *               ['cshKey']      => The CSH key for the field
     *               ['cshLabel']    => The code of the CSH label
     */
    protected function _getAdditionalFields(array &$taskInfo, $task, $schedulerModule)
    {
        $action = \tx_rnbase_util_TYPO3::isTYPO90OrHigher() ?
            $schedulerModule->getCurrentAction() : $schedulerModule->CMD;
        // Initialize extra field value
        if (!array_key_exists(FIELD_ITEMS, $taskInfo) || empty($taskInfo[FIELD_ITEMS])) {
            if ('add' == $action) {
                // New task
                $taskInfo[FIELD_ITEMS] = '';
            } elseif ('edit' == $action) {
                // Editing a task, set to internal value if data was not submitted already
                $taskInfo[FIELD_ITEMS] = $task->getAmountOfItems();
            } else {
                // Otherwise set an empty value, as it will not be used anyway
                $taskInfo[FIELD_ITEMS] = '';
            }
        }

        // Write the code for the field
        $fieldID = 'field_'.FIELD_ITEMS;
        // Note: Name qualifier MUST be "tx_scheduler" as the tx_scheduler's BE module is used!
        $fieldCode = '<input type="text" name="tx_scheduler['.FIELD_ITEMS.']" id="'.$fieldID.
                        '" value="'.$taskInfo[FIELD_ITEMS].'" size="10" />';
        $additionalFields = [];
        $additionalFields[$fieldID] = [
            'code' => $fieldCode,
            'label' => 'LLL:EXT:mksearch/locallang_db.xml:scheduler_indexTask_field_'.FIELD_ITEMS,
            'cshKey' => '_MOD_web_txschedulerM1',
        ];

        return $additionalFields;
    }

    /**
     * This method checks any additional data that is relevant to the specific task
     * If the task class is not relevant, the method is expected to return true.
     *
     * @param array                                                     $submittedData:   reference to the array containing the data submitted by the user
     * @param \TYPO3\CMS\Scheduler\Controller\SchedulerModuleController $schedulerModule: reference to the calling object (Scheduler's BE module)
     *
     * @return bool True if validation was ok (or selected class is not relevant), false otherwise
     */
    protected function _validateAdditionalFields(array &$submittedData, $schedulerModule)
    {
        return true;
    }

    /**
     * This method is used to save any additional input into the current task object
     * if the task class matches.
     *
     * @param array                           $submittedData: array containing the data submitted by the user
     * @param tx_mksearch_scheduler_ReindexRunTask $task:          reference to the current task object
     */
    protected function _saveAdditionalFields(array $submittedData, Tx_Rnbase_Scheduler_Task $task)
    {
        $task->setAmountOfItems($submittedData[FIELD_ITEMS]);
    }
}

if (defined('TYPO3_MODE') && $GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/mksearch/scheduler/class.tx_mksearch_scheduler_IndexTaskAddFieldProvider.php']) {
    include_once $GLOBALS['TYPO3_CONF_VARS'][TYPO3_MODE]['XCLASS']['ext/mksearch/scheduler/class.tx_mksearch_scheduler_IndexTaskAddFieldProvider.php'];
}
